@@ -3,9 +3,12 @@ from datetime import datetime, timedelta
 from enum import Enum
 from os import PathLike, makedirs
 from pathlib import Path
+import shutil
 from textwrap import indent
 from typing import Iterator, Tuple
 
+from dunamai import Version
+
 from .model.base import (
     ConfigurationEntry,
     ConnectionEntry,
@@ -23,6 +26,8 @@ from .utilities import get_logger
 
 LOGGER = get_logger('configuration')
 
+__version__ = Version.from_git().serialize(dirty=True)
+
 
 class Earth(ConfigurationEntry):
     """
@@ -298,9 +303,29 @@ class ConfigurationFile(ABC):
     def __getitem__(self, entry_type: type) -> [ConfigurationEntry]:
         return [entry for entry in self if isinstance(entry, entry_type)]
 
-    @abstractmethod
-    def write(self, directory: PathLike, overwrite: bool = False):
-        raise NotImplementedError
+    @property
+    def version_header(self) -> str:
+        return f'# `{self.name}` generated with NEMSpy {__version__}'
+
+    def write(
+        self, directory: PathLike, overwrite: bool = False, include_version: bool = True
+    ) -> Path:
+        output = f'{self}\n'
+        if include_version:
+            output = f'{self.version_header}\n{output}'
+
+        directory = ensure_directory(directory)
+        filename = directory / self.name
+
+        if filename.exists():
+            LOGGER.warning(
+                f'{"overwriting" if overwrite else "skipping"} existing file "{filename}"'
+            )
+        if not filename.exists() or overwrite:
+            with open(filename, 'w') as output_file:
+                output_file.write(output)
+
+        return filename
 
     @abstractmethod
     def __str__(self) -> str:
@@ -313,19 +338,6 @@ class ConfigurationFile(ABC):
 class NEMSConfigurationFile(ConfigurationFile):
     name = 'nems.configure'
 
-    def write(self, directory: PathLike, overwrite: bool = False):
-        directory = ensure_directory(directory)
-        filename = directory / self.name
-
-        if filename.exists():
-            LOGGER.warning(
-                f'{"overwriting" if overwrite else "skipping"} ' f'existing file "{filename}"'
-            )
-        if not filename.exists() or overwrite:
-            LOGGER.debug(f'writing NEMS configuration to "{filename}"')
-            with open(filename, 'w') as output_file:
-                output_file.write(str(self))
-
     @property
     def entries(self) -> [ConfigurationEntry]:
         return [self.sequence.earth, *self.sequence.models, self.sequence]
@@ -335,30 +347,12 @@ class NEMSConfigurationFile(ConfigurationFile):
             yield entry
 
     def __str__(self) -> str:
-        return (
-            '#############################################\n'
-            '####  NEMS Run-Time Configuration File  #####\n'
-            '#############################################\n'
-            '\n' + '\n'.join(f'# {entry.entry_type} #\n' f'{entry}\n' for entry in self)
-        )
+        return '\n'.join(f'# {entry.entry_type} #\n' f'{entry}\n' for entry in self).strip()
 
 
 class MeshFile(ConfigurationFile):
     name = 'config.rc'
 
-    def write(self, directory: PathLike, overwrite: bool = False):
-        directory = ensure_directory(directory)
-        filename = directory / self.name
-
-        if filename.exists():
-            LOGGER.warning(
-                f'{"overwriting" if overwrite else "skipping"} ' f'existing file "{filename}"'
-            )
-        if not filename.exists() or overwrite:
-            LOGGER.debug(f'writing mesh filenames to "{filename}"')
-            with open(filename, 'w') as output_file:
-                output_file.write(str(self))
-
     @property
     def entries(self) -> [ModelMeshEntry]:
         return [entry for entry in self.sequence if isinstance(entry, ModelMeshEntry)]
@@ -368,7 +362,7 @@ class MeshFile(ConfigurationFile):
             yield entry
 
     def __str__(self) -> str:
-        return '\n'.join([ModelMeshEntry.__str__(model_mesh) for model_mesh in self]) + '\n'
+        return '\n'.join([ModelMeshEntry.__str__(model_mesh) for model_mesh in self])
 
 
 class ModelConfigurationFile(ConfigurationFile):
@@ -379,57 +373,45 @@ class ModelConfigurationFile(ConfigurationFile):
         self.duration = duration
         super().__init__(sequence)
 
-    def write(self, directory: PathLike, overwrite: bool = False):
-        directory = ensure_directory(directory)
-        filename = directory / self.name
+    def write(
+        self, directory: PathLike, overwrite: bool = False, include_version: bool = True
+    ):
+        filename = super().write(directory, overwrite, include_version)
+        symbolic_link_filename = filename.parent / 'atm_namelist.rc'
 
-        if filename.exists():
-            LOGGER.warning(
-                f'{"overwriting" if overwrite else "skipping"} ' f'existing file "{filename}"'
-            )
-        if not filename.exists() or overwrite:
-            LOGGER.debug(f'writing model configuration to "{filename}"')
-            with open(filename, 'w') as output_file:
-                output_file.write(str(self))
-
-        symbolic_link_filename = directory / 'atm_namelist.rc'
         try:
             symbolic_link_filename.symlink_to(filename)
         except Exception as error:
             LOGGER.warning(f'could not create symbolic link: {error}')
-            with open(symbolic_link_filename, 'w') as output_file:
-                output_file.write(str(self))
+            shutil.copyfile(filename, symbolic_link_filename)
 
     def __str__(self) -> str:
         duration_hours = round(self.duration / timedelta(hours=1))
-        return (
-            '\n'.join(
-                [
-                    'total_member:            1',
-                    'print_esmf:              .true.',
-                    'namelist:                atm_namelist',
-                    f'PE_MEMBER01:             {self.sequence.processors}',
-                    f'start_year:              {self.start_time.year}',
-                    f'start_month:             {self.start_time.month}',
-                    f'start_day:               {self.start_time.day}',
-                    f'start_hour:              {self.start_time.hour}',
-                    f'start_minute:            {self.start_time.minute}',
-                    f'start_second:            {self.start_time.second}',
-                    f'nhours_fcst:             {duration_hours:.0f}',
-                    'RUN_CONTINUE:            .false.',
-                    'ENS_SPS:                 .false.',
-                    # 'dt_atmos:                   @[DT_ATMOS]'
-                    # 'atm_coupling_interval_sec:  @[coupling_interval_fast_sec]'
-                    #
-                    # 'iatm: @[IATM]'
-                    # 'jatm: @[JATM]'
-                    #
-                    # 'cdate0: @[CDATE]'
-                    # 'nfhout: @[NFHOUT]'
-                    # 'filename_base: @[FILENAME_BASE]'
-                ]
-            )
-            + '\n'
+        return '\n'.join(
+            [
+                'total_member:            1',
+                'print_esmf:              .true.',
+                'namelist:                atm_namelist',
+                f'PE_MEMBER01:             {self.sequence.processors}',
+                f'start_year:              {self.start_time.year}',
+                f'start_month:             {self.start_time.month}',
+                f'start_day:               {self.start_time.day}',
+                f'start_hour:              {self.start_time.hour}',
+                f'start_minute:            {self.start_time.minute}',
+                f'start_second:            {self.start_time.second}',
+                f'nhours_fcst:             {duration_hours:.0f}',
+                'RUN_CONTINUE:            .false.',
+                'ENS_SPS:                 .false.',
+                # 'dt_atmos:                   @[DT_ATMOS]'
+                # 'atm_coupling_interval_sec:  @[coupling_interval_fast_sec]'
+                #
+                # 'iatm: @[IATM]'
+                # 'jatm: @[JATM]'
+                #
+                # 'cdate0: @[CDATE]'
+                # 'nfhout: @[NFHOUT]'
+                # 'filename_base: @[FILENAME_BASE]'
+            ]
         )
 
 
